/* ser_sr24.h
 * 
 * Provides constant definitions, function declarations,
 * and other declarations for the use of the overall library.
 * This library is for the Winford Engineering serial I/O
 * board.
 * 
 * Copyright (C) 2000, Winford Engineering.
 * See file LICENSE.txt for details.
 * 
 * Winford Engineering
 * www.winfordeng.com
 */

/* This is the Windows version of the serial API header file.
 * This file defines those items that are needed by an
 * end-user application that makes use of this API.
 */

#ifndef ser_sr24_h
#define ser_sr24_h

#ifdef __cplusplus
extern "C"
{
#endif

/* This library uses standard calling convention, as does 
 * the Windows API.
 */
#define CALLCONVENTION __stdcall

/* Format for callback function, used with events */
typedef void (CALLCONVENTION *ser_eventfn)(int id, int value, void *extra);


/* Use a void pointer for the data structure, so all the needless details
 * don't have to be included in this file
 */
typedef void *ser_sr24;

/* structure used to specify how events should be handled */
typedef struct
{
	int type; /* which method to use for event notification */
	          /* 0 - none; disabled.
	           * 1 - callback function
	           * 2 - Windows message: sent to window ID w/ msg id
	           *     specified.  wparam will be the event id and
	           *     lparam will be the state (value).
	           * 3 - special callback, uses windows messages.
	           *     In this mode, the API creates its own window
	           *     and message processing function for that window,
	           *     and depends on the application to keep up with the
	           *     event loop.  The API's message processing function
	           *     then receives a pointer to data describing the event
	           *     and makes a callback to the application with that
	           *     data.  This has the advantage of being compatible with
	           *     Visual Basic because everything is done in VB's own
	           *     thread.
	           */
	ser_eventfn eventfn; /* used if type is 1 or 3. points to user
	                                     * defined function.
	                                     */
	void *window; /* if type is 2: window handle to send a message to */
	int msgid; /* If type is 2: message id to be sent */
	void *extra; /* If type is 1 or 3: extra data to be passed to the callback. */
} ser_event_spec;

/* Properties of the ports on the serial board. 
 * These values are used as the property parameter
 * for the ser_sr24_set_portmode() function
 */
#define SR24_BUFFER     0 /* on for enabled, off for disabled */
#define SR24_DIRECTION  1 /* on for output, off for input */
#define SR24_TYPE       2 /* on for analog, off for digital */

/* Values that a property can take on 
 * These are used as the mode parameter 
 * for the ser_sr24_set_portmode() function
 */
#define SR24_BUFFER_ENABLED          0
#define SR24_BUFFER_DISABLED         1
#define SR24_DIRECTION_INPUT         0
#define SR24_DIRECTION_OUTPUT        1
#define SR24_TYPE_DIGITAL            0
#define SR24_TYPE_ANALOG             1

/* Serial Baud Rates: ----------------------------------------------- */
/* All baud rates are listed, but the serial board only supports
 * 9600 or 19200 at this point
 */

#define SER_1200   9
#define SER_2400  11
#define SER_4800  12
#define SER_9600  13
#define SER_19200 14
#define SER_38400 15


/* Errors returned by functions: */
 
#define SER_SUCCESS                 0
#define SER_GENERAL_ERROR           -1
#define SER_SERIAL_ERROR            -10  // unable to open, read, or write to the serial port
#define SER_THREAD_ERROR            -11		 // General error ocurred in the threads library used by this API.
#define SER_NOT_SUPPORTED			-12  //returned if an API function is not supported by a device
#define SER_PIPE_ERROR				-13  // Error creating a pipe (in linux)
#define SER_READTHREAD_ERROR        -14  // Internal API error creating the reading thread or inside the reading thread
#define SER_EVENTTHREAD_ERROR       -15  // Internal API error creating the event thread of inside the event thread
#define SER_MALLOC_ERROR            -16  // Problem involving allocating memory
#define SER_WINDOWS_ERROR           -17  // Error specific to windows, such as creating a window, or window class

#define SER_INVALID_HANDLE          -101 //No device is currently open with that handle
#define SER_INVALID_MODEL           -102 //Model was invalid (doesn't exist)
#define SER_INVALID_PORT            -104 //Port specified is invalid for the device model
#define SER_INVALID_PORTMODE        -105 //There is no such mode for that port, or that mode is currently unavailable
#define SER_INVALID_PORTVALUE       -106 //Value passed to output function is out of valid range
#define SER_INVALID_EVENTID			-107 //Cannot assign that id value to an event
#define SER_INVALID_EVENTTYPE		-108 //Not one of the supported event types
#define SER_INVALID_BIT				-109 //Value passed identifying bit is out of range
#define SER_INVALID_PROPERTY        -110 //Not a valid property for the port specified
#define SER_INVALID_CHANNEL         -111 //Invalid analog channel number specified

#define SER_SERIALPORT_CONFLICT     -114 //Serial port specified is already in use
#define SER_DEVICE_NOTFOUND         -115 //Device is not present on the specified serial port, or is not powered up
#define SER_WRONG_PORTMODE          -150 //Device is not in the correct mode.
#define SER_EVENT_CONFLICT			-151 //This error likely occurs if a bit event is registered on a port whose port event is enabled

#define SER_TIMEOUT                 -201 //Timeout ocurred. Most likely communication has been lost w/ device	



/* Functions: the core of the API */
ser_sr24 CALLCONVENTION ser_sr24_open(char *serial, unsigned int baud, int *result);
int CALLCONVENTION ser_sr24_verify_connection(ser_sr24 handle);
int CALLCONVENTION ser_sr24_reset_board(ser_sr24 handle);
int CALLCONVENTION ser_sr24_output_byte(ser_sr24 handle, int port, int value);
int CALLCONVENTION ser_sr24_input_byte(ser_sr24 handle, int port, int *value);
int CALLCONVENTION ser_sr24_output_bit(ser_sr24 handle, int port, int bit, int value);
int CALLCONVENTION ser_sr24_input_bit(ser_sr24 handle, int port, int bit, int *value);
int CALLCONVENTION ser_sr24_input_analog(ser_sr24 handle, int channel, int *value);
int CALLCONVENTION ser_sr24_get_portmode(ser_sr24 handle, int port, int property);
int CALLCONVENTION ser_sr24_set_portmode(ser_sr24 handle, int port, int property, int mode);
int CALLCONVENTION ser_sr24_set_timeout(ser_sr24 handle, unsigned int timeout);
int CALLCONVENTION ser_sr24_register_event(ser_sr24 handle, int port, int bit, int id);
int CALLCONVENTION ser_sr24_set_event_handler(ser_sr24 handle, ser_event_spec *event);
int CALLCONVENTION ser_sr24_wait_event(ser_sr24 handle, int *events, unsigned int length, unsigned int timeout, int *value);
int CALLCONVENTION ser_sr24_close(ser_sr24 handle);
char* CALLCONVENTION ser_sr24_error_string(int errorcode);

#ifdef __cplusplus
}
#endif

#endif 
/* end of ser_sr24_h */
